/*
 * Mp3Splt -- Utliity for mp3/ogg splitting without decoding
 *
 * Copyright (c) 2002-2003 M. Trotta - <matteo.trotta@lib.unimib.it>
 *
 * http://mp3splt.sourceforge.net
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 * Tested on: Linux 2.4.18 #4 Fri May 31 01:25:31 PDT 2002 i686
 */

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <string.h>
#include <ctype.h>
#include <vorbis/codec.h>
#include <vorbis/vorbisfile.h>

#include "mp3.h"
#include "wrap.h"
#include "ogg.h"
#include "splt.h"
#include "mp3splt.h"
#include "cddb.h"

const char *chan[] = {
	"Stereo",
	"Joint Stereo",
	"Dual Mono",
	"Mono",
};

const char *genre_list[] = {
"Blues",                 "Classic Rock",          "Country",
"Dance",                 "Disco",                 "Funk",
"Grunge",                "Hip-Hop",               "Jazz",
"Metal",                 "New Age",               "Oldies",
"Other",                 "Pop",                   "R&B",
"Rap",                   "Reggae",                "Rock",
"Techno",                "Industrial",            "Alternative",
"Ska",                   "Death Metal",           "Pranks",
"Soundtrack",            "Euro-Techno",           "Ambient",
"Trip-Hop",              "Vocal",                 "Jazz+Funk",
"Fusion",                "Trance",                "Classical",
"Instrumental",          "Acid",                  "House",
"Game",                  "Sound Clip",            "Gospel",
"Noise",                 "Alt. Rock",             "Bass",
"Soul",                  "Punk",                  "Space",
"Meditative",            "Instrum. Pop",          "Instrum. Rock",
"Ethnic",                "Gothic",                "Darkwave",
"Techno-Indust.",        "Electronic",            "Pop-Folk",
"Eurodance",             "Dream",                 "Southern Rock",
"Comedy",                "Cult",                  "Gangsta",
"Top 40",                "Christian Rap",         "Pop/Funk",
"Jungle",                "Native American",       "Cabaret",
"New Wave",              "Psychadelic",           "Rave",
"Showtunes",             "Trailer",               "Lo-Fi",
"Tribal",                "Acid Punk",             "Acid Jazz",
"Polka",                 "Retro",                 "Musical",
"Rock & Roll",           "Hard Rock",             "Folk",
"Folk/Rock",             "National Folk",         "Swing",
"Fusion",                "Bebob",                 "Latin",
"Revival",               "Celtic",                "Bluegrass",
"Avantgarde",            "Gothic Rock",           "Progress. Rock",
"Psychadel. Rock",       "Symphonic Rock",        "Slow Rock",
"Big Band",              "Chorus",                "Easy Listening",
"Acoustic",              "Humour",                "Speech",
"Chanson",               "Opera",                 "Chamber Music",
"Sonata",                "Symphony",              "Booty Bass",
"Primus",                "Porn Groove",           "Satire",
"Slow Jam",              "Club",                  "Tango",
"Samba",                 "Folklore",              "Ballad",
"Power Ballad",          "Rhythmic Soul",         "Freestyle",
"Duet",                  "Punk Rock",             "Drum Solo",
"A Capella",             "Euro-House",            "Dance Hall",
"Goa",                   "Drum & Bass",           "Club-House",
"Hardcore",              "Terror",                "Indie",
"BritPop",               "Negerpunk",             "Polsk Punk",
"Beat",                  "Christian Gangsta Rap", "Heavy Metal",
"Black Metal",           "Crossover",             "Contemporary Christian",
"Christian Rock",        "Merengue",              "Salsa",
"Trash Metal",           "Anime",                 "Jpop",
"Synthpop"};

splt_state *splt_state_new (splt_state *state)
{
	if ((state = (splt_state *) malloc(sizeof(splt_state)))==NULL) {
		perror("malloc");
		exit(1);
	}
	memset(state, 0x0, sizeof(splt_state));
	state->mstate = NULL;
	state->ostate = NULL;
	
	return state;
}

void splt_state_free (splt_state *state)
{
	if (state!=NULL) {
		if (state->ostate) { // If file has been opened with ov_open we should clse it using ov_clear()
			ov_clear(&state->ostate->vf);
			v_free(state->ostate);
		}
		else if (state->mstate) {
			fclose(state->mstate->file_input);
			mp3_state_free(state->mstate);
		}
		free(state);
	}
}

void splt_abort(char *msg, splt_state *state)
{
	splt_state_free(state);
	error(msg);
}

int main (int argc, char *argv[]) {

	FILE *file_input = NULL;
	unsigned char filename[512];
	int tracks = 0, i = 0, j = 0;
	short framemode=0, wrapoption=0, cddboption=0, listoption=0, errsyncoption = 0, outputdir = 0;
	short noid3=0, quietoption=0, ogg=0, outputfile=0, status = 0, silenceoption = 0, zpad=0, outformat=0;
	char option, *ptr = NULL, id3buffer[128], *carg=NULL, *darg=NULL, *oarg=NULL;
	splt_state *state = NULL;
	
	fprintf (stderr, NAME" "VER" "YEAR" by "AUTHOR" "EMAIL"\n");
	fprintf (stderr, "THIS SOFTWARE COMES WITH ABSOLUTELY NO WARRANTY! USE AT YOUR OWN RISK!\n");

	while ((option=getopt(argc, argv, "wlesfqnc:d:o:"))!=EOF) {
	  switch (option) {
		case 'f': framemode=1;
			  break;
		case 'w': wrapoption=1;
			  break;
		case 'l': listoption=1;
			  wrapoption = 1;
		   	  break;
		case 'e': errsyncoption=1;
			  break;
		case 's': silenceoption=1;
			  framemode=1;
			  break;
		case 'c': cddboption=1;
			  carg = argv[optind-1];
			   break;
		case 'd': outputdir=1;
		  	  darg = argv[optind-1];
			  break;
		case 'o': outformat=1;
			  oarg = argv[optind-1];
			  break;
		case 'n': noid3=1;
			  break;
		case 'q': quietoption = 1;
			  break;
		default: error("Run without arguments for HELP. Read man page for complete documentation");
	  }
	}

	if (optind > 1)
	{
		argv = rmopt(argv, optind, argc);
		argc -= optind-1;
	}
	
	if ((argc<4)&&(!wrapoption)&&(!cddboption)&&(!errsyncoption)&&(!silenceoption))
	{
	   fprintf (stdout, "USAGE (Read man page for complete documentation)\n");
	   fprintf (stdout, "      mp3splt [options] FILE [begin_time] [end_time...] [outputfile]\n");
	   fprintf (stdout, "      TIME FORMAT: min.sec[.0-99], even if minutes are over 59. \n");
	   fprintf (stdout, "DESCRIPTION\n");
	   fprintf (stdout, "      Mp3Splt splits an mp3/ogg file from a begin to an end time, or files\n");
	   fprintf (stdout, "      created with either Mp3Wrap or AlbumWrap. Or it can get splitpoints\n");
	   fprintf (stdout, "      and filenames automatically from internet or local files (see -c).\n");
	   fprintf (stdout, "      NOTE: To split VARIABLE BITRATE MP3 you must use -f (see below).\n");
	   fprintf (stdout, "OPTIONS\n");
	   fprintf (stdout, " -w   Splits wrapped files created with Mp3Wrap or AlbumWrap.\n");
	   fprintf (stdout, " -l   Lists the tracks from file without extraction. (Only for wrapped mp3)\n");
	   fprintf (stdout, " -e   Error mode: split mp3 with sync error detection. (For concatenated mp3)\n");
	   fprintf (stdout, " -s   Split with silence detection: it scans file to find silence positions\n");
	   fprintf (stdout, "      and splits all or a user NUMBER (specified after filename) tracks.\n");
	   fprintf (stdout, " -f   Frame mode (mp3 only): process all frames. For higher precision and VBR.\n");
	   fprintf (stdout, " -c + file.cddb, file.cue or \"query\". Get splitpoints and filenames from a\n");
	   fprintf (stdout, "      .cddb or .cue file or from Internet (\"query\"). Use -sc to auto-adjust.\n");
	   fprintf (stdout, " -o + FORMAT: output name containing @a artist, @b album, @t title, @n number.\n");
	   fprintf (stdout, " -d + DIRNAME: to put all output files in the directory DIRNAME.\n");
	   fprintf (stdout, " -n   No Tag: does not write ID3v1 or vorbis comment. If you need clean files.\n");
	   fprintf (stdout, " -q   Quiet mode: do not prompt for anything and print less messages.\n");
	   exit (1);
	}
	else {
		if (wrapoption && ((argc<2)||cddboption))
			error("wrap usage is 'mp3splt -w album_MP3WRAP.mp3'");

		if (errsyncoption && ((argc<2)|| cddboption || wrapoption))
			error("SyncError usage is 'mp3splt -e FILE'");
		
		if (cddboption && ((argc<2)||wrapoption))
			error("CDDB usage is 'mp3splt -[s]c SOURCE FILE [GAP]''");
		
		if (silenceoption && ((argc<2)||wrapoption))
			error("usage is 'mp3splt -s FILE [NUMBER]'");
		
		if (outformat && (!cddboption))
			fprintf(stderr, "Warning: output format can be used only with -c option!\n");
	}
	
	if (!(file_input=fopen(argv[1], "rb"))) {
		perror(argv[1]);
		exit(1);
	}

	if (!wrapoption) {
		ptr = strtoupper(argv[1]); // Assure that any combination of these string will be found. You should not modify them!
		if ((strstr(ptr, WRAP)!=NULL)||(strstr(ptr, ALBW)!=NULL)) {
			fprintf (stderr, "\nWARNING: "WRAP" or "ALBW" string found in filename. Switching to wrap mode...\n");
			fprintf (stderr, "\t Remove string if program gives errors or if you want standard mode.\n\n");
			wrapoption = 1;
			cddboption=0;
		}
		else {
			// Is this an ogg file?
			if ((strstr(ptr, OGGEXTU))!=NULL)
				ogg = 1;
				
			if (errsyncoption && ogg)
				error("can't use error option with ogg!");

			state = splt_state_new(state);

			memset (filename, '\0', 512);
			state->id.genre = 0xFF;
			state->id.year[0]='\0';
			
			if (cddboption) {
				if ((strstr(carg, ".cue")!=NULL)||(strstr(carg, ".CUE")!=NULL)) {
					strncpy(filename, carg, 511);
					tracks = get_cue_splitpoints(filename, state);
				}
				else {
					if (strcmp(carg, "query")==0) {
						fprintf (stderr, "CDDB QUERY. Insert album and artist informations to find cd.\n");
#ifdef _WIN32
						i = search_freedb(state, argv[0]);
#else
						i = search_freedb(state);
#endif
						if (i==0) {
							strncpy(filename, CDDBFILE, strlen(CDDBFILE));
							fprintf (stderr, "Now starting to split file...\n\n");
						}
						else {
							switch(i) {
								case -1: fprintf (stderr, "Error: No cd found in this search!\n");
									break;
								case -2: fprintf (stderr, "Error: An error occurred while reading Freedb informations\n");
									break;
								case -3: fprintf (stderr, "Error: Selected Cd not found on "FREEDB". Try another cd.\n");
									break;
								case -4: fprintf (stderr, "Error: Bad communication or server cowardly refused to give informations!\n");
									break;
								case -5: fprintf (stderr, "Error: An error occurred while trying to read server answer.\n");
									break;
								default: break;
							}
							splt_abort("", state);
						}
					}
					else strncpy(filename, carg, 511);
					tracks = get_cddb_splitpoints(filename, state);
				}
				if (tracks<=0){
					fprintf (stderr, "%s: this is not a valid cddb or cue file, try another.\n", filename);
					splt_abort("", state);
				}
				strncpy(state->performer[0], state->id.album, 128);
				cleanstring(state->performer[0]);
				if (outformat)
				{
					i=parse_outformat(oarg, state->format);
					switch (i) {
						case -1: splt_abort("", state);
									break;
						case 1: splt_abort("your output format may be ambiguous! (@t or @n missing)", state);
								  break;
						case 0: break;
					}
				}
			}
			else tracks=0;
		}
		free(ptr);
	}
	
	if (outputdir)
#ifdef _WIN32
		mkdir(darg);
#else
		mkdir(darg, 0755);
#endif
	
	if (wrapoption) {
		if (dewrap(file_input, listoption, quietoption, darg)!=0) {
			fflush(stdout);
			error("file is not wrapped or might be damaged! Try -e option.");
		}
		else if (listoption) {
				fprintf(stdout, "\n\n");
				fflush(stdout);
			}
			else fprintf (stderr, "\n\nAll files have been splitted correctly. Visit http://mp3wrap.sourceforge.net!\n");

		fclose(file_input);
	}
	else
	{
		if (!cddboption && !errsyncoption && !silenceoption) {
			state->splitpoints[0] = c_seconds(argv[2]); // calculate seconds from min.sec command line
			state->splitpoints[1] = c_seconds(argv[3]);
			
			if (state->splitpoints[0]==-1 || state->splitpoints[1]==-1)
				splt_abort("wrong time format. Type man mp3splt to see complete documentation.", state);

			if (state->splitpoints[0]>=state->splitpoints[1])
				splt_abort("not enough seconds!", state);
				
			zpad = dot_pos(argv[2], zpad);
			zpad = dot_pos(argv[3], zpad);
				
			if (argc>4) {
				if ((state->splitpoints[2] = c_seconds(argv[4]))==-1)
					outputfile = 1;
				else {
					zpad = dot_pos(argv[4], zpad);
					if (state->splitpoints[1] < state->splitpoints[2]) {
						tracks = 2;
						for (i=5; (i<argc) && (tracks < MAXTRACKS); i++) {
							state->splitpoints[tracks+1] = c_seconds(argv[i]);
							zpad = dot_pos(argv[i], zpad);
							if ((state->splitpoints[tracks+1] == -1) || (state->splitpoints[tracks] >= state->splitpoints[tracks+1]))
							{
								fprintf (stderr, "Warning: omitting '%s' (bad time format or not enough seconds)\n", argv[i]);
								break;
							}
							else tracks++;
						}
					}
				}
			}
		}
		
		i = 0;
		
		if (ogg) {
			
			if (!(state->ostate = ogginfo(file_input, state->ostate)))
				splt_abort("input does not appear to be a valid ogg vorbis bitstream", state);
			
			i = (int) ov_time_total(&state->ostate->vf, -1); // Total Seconds
			j = (int) (i / 60); // Minutes
			fprintf(stderr, "Ogg Vorbis Stream - %ld - %ld Kb/s - %d channels", state->ostate->vd->vi->rate, 
				state->ostate->vd->vi->bitrate_nominal/1024, state->ostate->vd->vi->channels);
			vorbis_comment_init(&state->ostate->vc);
		}
		else {
			struct mp3 *mfile;
			if (!(state->mstate = mp3info(file_input, state->mstate, quietoption, framemode)))
				splt_abort("input does not appear to be a valid mp3 stream", state);
			
			if (state->mstate->framemode) {
				state->mstate->mp3file.fps = (state->mstate->mp3file.freq*(2-state->mstate->mp3file.mpgid));
				state->mstate->mp3file.fps /= PCM;
			}
			else state->mstate->mp3file.bitrate = (state->mstate->mp3file.firsthead.bitrate/BYTE)*1000;

			mfile = &state->mstate->mp3file; // For short lines... ;-)
			
			fprintf (stderr, "MPEG %d Layer %d - %d Hz - %s", (2-mfile->mpgid), mfile->layer, mfile->freq, chan[mfile->channels]);
			if (state->mstate->framemode) fprintf (stderr, " - FRAME MODE");
			else fprintf (stderr, " - %d Kb/s", (mfile->bitrate*BYTE)/1000);
			if ((!state->mstate->framemode)&&(mfile->len > 0)) {
				i = (int)((mfile->len-mfile->firsthead.ptr)/mfile->bitrate);
				j = (int)(i/60);
			}
			else if (mfile->xing != 0) {
					i = (int)(state->mstate->frames/mfile->fps);
					j = (int)(i/60);
					state->mstate->frames = 0;
			}
		}

		if (i!=0) fprintf (stderr, " - Total time: %dm.%02ds", j, i%60);
		fprintf (stderr, "\n");
		
		if (errsyncoption) {
			i = syncerror_split(state->mstate, quietoption, darg);
			if (i!=0)
				status = 1;
			switch (i) {
				case -1: splt_abort("an error occurred while processing file!", state);
						  break;
				case 0: break;
				case 1: splt_abort("sorry, no sync errors found. Can't split file.", state);
						  break;
				case 2: splt_abort("too many sync errors! Are you sure this is a valid mp3 stream?", state);
						  break;
				case 3: fprintf(stderr, "Aborted.\n");
						  break;
			}
		}
		else
		if (silenceoption && !cddboption) {
			FILE *log;
			int stracks = 0, found = 0;
			struct ssplit *temp, **list;
			if (argc > 2) {
				for (i = 0; i<strlen(argv[2]); i++) {
					if (isdigit(argv[2][i])==0) {
						fprintf(stderr, "Error: bad track argument '%s'. Usage is 'mp3splt -s FILE [NUMBER]'\n", argv[3]);
						splt_abort("", state);
					}
				}
				stracks = atoi(argv[2]);
			}
			else fprintf(stderr, "Silence split type: working in auto mode. \n");
			
			if (ogg)
				list = &state->ostate->silence_list;
			else list = &state->mstate->silence_list;
			
			if ((log=fopen(SSPLITLOG, "r"))) {
				short error = 0;
				if (fgets(filename, 512, log)!=NULL) {
					filename[strlen(filename)-1]='\0';
					if (strcmp(filename, argv[1])!=0)
						error = 1;
				}
				else error = 1;
				if (error) {
					fclose(log);
					log = NULL;
				}
			}
			if (log == NULL) {
				fprintf(stderr, "\tDetecting silence positions, please wait...\r");
				if (ogg)
					found = ogg_scan_silence(state->ostate, 0, DEFAULTOGGTS, 1, NULL, 0);
				else found = mp3_scan_silence(state->mstate, state->mstate->mp3file.firsthead.ptr, 0, DEFAULTMP3TS, 1);
				if (found > 0) {
					if (!(log=fopen(SSPLITLOG, "w")))
						fprintf(stderr, "Warning: can't write "SSPLITLOG" file to save informations!\n");
					else {
						temp = *list;
						fprintf(log, "%s\n", argv[1]);
						while (temp != NULL) {
							fprintf(log, "%f\t%d\n", temp->position, temp->len);
							temp = temp->next;
						}
						fclose(log);
					}
				}
			}
			else {
				fprintf(stderr, SSPLITLOG" file found! Reading silence points from file to save time ;)\n");
				found = parse_ssplit_file(list, log);
				fclose(log);
			}
			
			if (found <= 0) {
				fprintf(stderr, "Error: sorry, no silence points found in this file!\n");
				status=1;
			}
			
			if (!status) {
				fprintf (stderr, "Total silence points found: %d. ", found);
				found++;
				if ((stracks > 0)&&(stracks < MAXTRACKS)) {
					if (stracks < found)
						found = stracks;
				}
				fprintf(stderr, "Selected %d tracks...\n", found);
				state->splitpoints[0] = 0.f;
				temp = *list;
				for (i = 1; i < found; i++) {
					if (temp == NULL) {
						found = i;
						break;
					}
					state->splitpoints[i] = temp->position;
					temp = temp->next;
				}
				order_splitpoints(state->splitpoints, found);
				state->splitpoints[found] = -1.f;
				
				for (i = 0; i < found; i++) {
					sprintf(filename, "Track %02d", i+1);
					if (outputdir)
					{
						char temp[512];
						strncpy(temp, filename, 512);
						sprintf(filename, "%s%c%s", darg, DIRCHAR, temp);
					}
					if (ogg)
						strncat(filename, OGGEXT, 4);
					else strncat(filename, MP3EXT, 4);
					fprintf (stderr, "\n %3d %% -> Splitting %s... ", (int)((i+1)*100)/found, filename);
					if (ogg) j = oggsplit(filename, state->ostate, state->splitpoints[i], state->splitpoints[i+1], 0);
					else j = mp3split (filename, state->mstate, NULL, state->splitpoints[i], state->splitpoints[i+1], 0);
					if (j == 0)
						fprintf (stderr, "OK");
					else {
						fprintf (stderr, "FAILED\nError: an error occurred!");
						status=1;
					}
				}
				fprintf (stderr, "\n\n");
			}
		}
		else {
			if (!quietoption && cddboption) {
				fprintf (stderr, "Reading informations from %s...\n", filename);
				fprintf (stderr, "Tracks: %d \n", tracks);
				fprintf (stderr, "Artist: %s\n", state->id.artist);
				fprintf (stderr, "Album: %s\n\n", state->id.album);
			}

			if (silenceoption) {
				silenceoption = DEFAULT_GAP;
				if (argc > 2) {
					for (i = 0; i<strlen(argv[2]); i++) {
						if (isdigit(argv[2][i])==0) {
							fprintf(stderr, "Warning: bad gap argument '%s'. Using default: %d\n", argv[2], DEFAULT_GAP);
							silenceoption = 0;
							break;
						}
					}
					if (silenceoption != 0)
						silenceoption = (short) atoi(argv[2]);
					else silenceoption = DEFAULT_GAP;
				}
			}
			
			if (silenceoption && !quietoption) fprintf (stderr, "Working in Frame mode with SILENCE AUTO-ADJUST, please wait a little more...\n\n");
			
			j = 1;

			do 	{

				memset (filename, '\0', 512);
				ptr = argv[1];

				while (strchr(ptr, DIRCHAR)!=NULL) // Will create file where we are (PWD)
					ptr = strchr(ptr, DIRCHAR) + 1;

				if (!cddboption) {
					if (outputfile)
						strncpy(filename, argv[4], 511);
					else {
						char zstart[32], zend[32];
						if ((strlen(ptr)>4) && (((strstr(ptr, MP3EXT))!=NULL) || (ogg)))
							strncpy (filename, ptr, strlen(ptr)-4);
						else strncpy (filename, ptr, 511);
						zero_pad(argv[1+j], zstart, zpad);
						zero_pad(argv[2+j], zend, zpad);
						sprintf (filename, "%s"SEP"%s"SEP"%s", filename, zstart, zend);
					}
				}
				else {
					memset(state->id.title, 0x00, 31);
					strncpy(state->id.title, state->fn[j], 30);
					if (outformat)
					{
						char temp[256], fm[256];
						for (i=0; i<OUTNUM; i++) {
							memset(temp, 0, 256);
							memset(fm, 0, 256);
							if (strlen(state->format[i])==0) break;
							if (state->format[i][0]=='%') {
								temp[0]='%';
								temp[1]='s';
								switch (state->format[i][1])
								{
									case 'a': 
												sprintf(temp+2, state->format[i]+2);
												sprintf(fm, temp, state->id.artist);
												break;
									case 'b':
												sprintf(temp+2, state->format[i]+2);
												sprintf(fm, temp, state->performer[0]);
												break;
									case 't':
												sprintf(temp+2, state->format[i]+2);
												sprintf(fm, temp, state->fn[j]);
												break;
									case 'p':
												sprintf(temp+2, state->format[i]+2);
												sprintf(fm, temp, state->performer[j]);
												break;
									case 'n':
												temp[1]='0';
												temp[2]='2';
												temp[3]='d';
												sprintf(temp+4, state->format[i]+2);
												sprintf(fm, temp, j);
												break;
								}
							}
							else strncpy(fm, state->format[i], MAXOLEN);
							strncat(filename, fm, strlen(fm));
						}
					}
					else sprintf (filename, "%s - %02d - %s", state->id.artist, j, state->fn[j]);
				}

				if (!outputfile) {
					if (ogg)
						strncat(filename, OGGEXT, 4);
					else strncat(filename, MP3EXT, 4);
				}

				if (ogg) vorbis_comment_init(&state->ostate->vc);

				if (!noid3) {
					if (!cddboption) {
						strncpy(state->id.title, ptr, 30);
						strncpy(state->id.album, "Splitted by "NAME" v. "VER, 30);
						strncpy(state->id.artist, "From \0", 6);
						strncat(state->id.artist, argv[1+j], 31-strlen(state->id.artist));
						strncat(state->id.artist, " To \0", 31-strlen(state->id.artist));
						strncat(state->id.artist, argv[2+j], 31-strlen(state->id.artist));
						strncpy(state->id.year, "2003\0", 5);
					}

					state->id.title[30]='\0';
					state->id.artist[30]='\0';
					state->id.album[30]='\0';

					if (tracks>0) i=j;
					else i=0x00;

					ptr = NULL;

					if (ogg) {
						char *gptr;
						if (state->id.genre == 0xFF) // Out of genre list
							gptr = NULL;
						else gptr = (char *) genre_list[(int) state->id.genre];
						ptr = trackstring(i);
						v_comment(&state->ostate->vc, state->id.artist, state->id.album, state->id.title, ptr, state->id.year, gptr, "http://mp3splt.sourceforge.net");
						if (ptr!=NULL) free(ptr);
					}
					else ptr = id3(id3buffer, state->id.title, state->id.artist, state->id.album, state->id.year, state->id.genre, "http://mp3splt.sf.net", i);
				}
				else ptr = NULL;

				if (tracks > 1)
					fprintf (stderr, " %3d %% -> ", (j*100)/tracks);

				if (outputdir)
				{
					char temp[512];
					strncpy(temp, filename, 512);
					sprintf(filename, "%s%c%s", darg, DIRCHAR, temp);
				}

				if (!cddboption) fprintf (stderr, "Splitting ");
				fprintf (stderr, "%s... ", filename);
				fflush(stderr);
				
				if (ogg) i = oggsplit(filename, state->ostate, state->splitpoints[j-1], state->splitpoints[j], silenceoption);
				else i = mp3split (filename, state->mstate, ptr, state->splitpoints[j-1], state->splitpoints[j], silenceoption);

				switch (i) {
					case 0:	fprintf (stderr, "OK\n");
							break;
					case -1:	fprintf (stderr, "FAILED!\n");
							if (cddboption)
								splt_abort("file is damaged or this is not the right cddb!", state);
							else splt_abort("begin is out of file or not enough seconds!", state);
							break;
					case -2:	fprintf (stderr, "OK\nWarning: an error occurred while seeking input file. Trying to continue...\n");
				}

			}  while (j++<tracks);

			if (!quietoption) {
				if (cddboption) {
					fprintf (stderr, "\nAll files successfully splitted! ");
					if (ogg) fprintf (stderr, "\n");
				}
				if (!ogg)
					if (state->mstate->framemode) 
						fprintf (stderr, "Processed %lu frames - Sync errors: %lu", state->mstate->frames, state->mstate->syncerrors);

				if (cddboption &&!silenceoption)
				{
					fprintf (stderr, "\n\n+-----------------------------------------------------------------------------+\n");
					fprintf (stderr, "|NOTE: When you use CDDB, splitted files might be not very precise due to:    |\n");
					fprintf (stderr, "|1) Who extracts CD tracks might use \"Remove silence\" option. This means that |\n");
					fprintf (stderr, "|   the large mp3 file is shorter than CD Total time. Never use this option.  |\n");
					fprintf (stderr, "|2) Who burns CD might add extra pause seconds between tracks.  Never do it.  |\n");
					fprintf (stderr, "|3) Encoders might add some padding frames so  that  file is longer than CD.  |\n");
					fprintf (stderr, "|4) There are several entries of the same cd on CDDB, find the best for yours.|\n");
					fprintf (stderr, "|   Usually you can find the correct splitpoints for your mp3, so good luck!  |\n");
					fprintf (stderr, "+-----------------------------------------------------------------------------+\n");
					fprintf (stderr, "| TRY TO ADJUST SPLIT POINTS WITH -sc OPTION. Read man page for more details! |\n");
					fprintf (stderr, "+-----------------------------------------------------------------------------+\n\n");
				}
				else if (!ogg)
						if (state->mstate->framemode) fprintf (stderr, "\n");
			}
		}
	}

	splt_state_free(state);
	
	return status;
}

